#include<stdlib.h>
#include<stdio.h>
#include<stdbool.h>

#include "algoKruskal.h"
#include "graphes.h"

typedef struct arc_t {
  int from;
  eltadj* to;            
} arc;

sommet* getSomByLabel(graphe* g, int label);
void dfsCycle(graphe* g, sommet* s, sommet* previous, bool* marked, int labelToFind, bool* result);
int comp(const void** a,const void** b);


sommet* getSomByLabel(graphe* g, int label) {
  sommet* currS = g->premierSommet;
  while (currS != NULL) {
    if (currS->label == label)
      return currS;
    else
      currS = currS->suivant;
  }

  return NULL;
}

/* Idée : On explore tous les chemins possibles partant de s en utilisant 
 * l'algorithme DFS. Si pendant qu'on réalise ceci, il arrive un moment où 
 * l'algorithme regarde un sommet qui est déjà marqué MAIS dont le label est 
 * celui du noeud de départ sans que celui-ci soit le noeud précédent du 
 * chemin en cours, cela veut dire qu'on a trouvé un cycle ! */
void dfsCycle(graphe* g, sommet* s, sommet* previous, bool* marked, int labelToFind, bool* result) {
  marked[s->label] = true;
  eltadj* currA = s->adj;

  // Si on a trouvé un cycle, plus besoin de continuer le parcours !
  while (currA != NULL && !(*result)) {
    sommet* destS = getSomByLabel(g, currA->dest);

    if (!marked[destS->label])
      dfsCycle(g, destS, s, marked, labelToFind, result);
    else if (labelToFind == destS->label && previous != destS)
      *result = true;

    currA = currA->suivant;
  }
}

int estAcyclique(graphe *g) {
  if (g == NULL || g->nbS < 3) return 0;

  sommet* currS = g->premierSommet;
  while (currS != NULL) {
    bool* marked = (bool*)calloc(g->maxS + 1, sizeof(bool));
    bool result = false;
    dfsCycle(g, currS, NULL, marked, currS->label, &result);
    free(marked);
    if (result)
      return 1;

    currS = currS->suivant;
  }

  return 0;
}


// Fonction de comparaison utilisée pour trier les arcs
int comp(const void** a,const void** b) {
  arc const *pa = *a;
  arc const *pb = *b;
  if (pa->to->poids == pb->to->poids) 
    return 0;
  if (pa->to->poids < pb->to->poids)
    return -1;
  return 1;
}


void algoKruskal(graphe *g, graphe *h) {
  // Remplir un tableau avec les arcs
  arc* arcsTab[g->nbA];
  int i = 0;
  sommet* psommet = g->premierSommet;
  eltadj* padj;
  do {
    if (psommet->adj != NULL) {
      padj = psommet->adj;
      do
      {
        arc* parc = (arc*) malloc(sizeof(arc));
        parc->from = psommet->label;
        parc->to = padj;
        arcsTab[i] = parc;
        i++;
        padj = padj->suivant;
      }
      while (padj != NULL);
    }
    psommet = psommet->suivant;
  }	while (psommet != NULL); 
  
  /* Tri des arcs par poids croissants */
  qsort(arcsTab, sizeof arcsTab / sizeof *arcsTab, sizeof *arcsTab, comp);
  printf("\n");
  
  // Ajout de tous les sommets à l'arbre
  for (int i = 0; i < g->nbS; i++)
    ajouterSommet(h, 0);
  
  // Ajout des arcs
  for (int i = 0; i < g->nbA; i++) { 
    ajouterArc(h, arcsTab[i]->from, arcsTab[i]->to->dest, arcsTab[i]->to->poids);

    // L'arbre ajouté crée-t-il un cycle ? Si oui, on le supprime
    if (estAcyclique(h) != 0)
      supprimerArc(h, arcsTab[i]->from, arcsTab[i]->to->dest);
  }
  
  // Libération de la mémoire
  for (int i = 0; i < g->nbA; i++) 
    free(arcsTab[i]);  
}

