#ifndef graphes_h
#define graphes_h

/*
structure pour les éléments de la liste des adjacents à un sommet du graphe
*/
struct eltadj_t{
	int dest;
	int poids;            
	struct eltadj_t *suivant;
};

/*
structure pour représenter un sommet du graphe
*/
struct sommet_t{
	int label;
	int info;    
	struct sommet_t *suivant;      
	struct eltadj_t *adj;  
};

/*
structure pour représenter un graphe
*/
struct graphe_t{
	int nbS;          
	int nbA;         
	int maxS;        
	struct sommet_t *premierSommet;
	struct sommet_t *dernierSommet; 
};

typedef struct graphe_t graphe; 
typedef struct sommet_t sommet;
typedef struct eltadj_t eltadj;

/*
- Recoit l'adresse d’une variable de type graphe. 
- La variable correspondante sera initialisée au graphe vide.
*/
void initialiserGraphe(graphe *);

/*
- Recoit l'adresse d’une variable de type graphe et un entier. 
- Le champ maxS est incrémenté d’une unité, un sommet de label maxS (après incrémentation) 
est ajouté ́au graphe en fin de liste et ce sommet porte l’information "info" fournie en argument.
- Retourne le code d'erreur -1 si l’allocation de mémoire n’est pas possible
*/
int ajouterSommet(graphe *, int info);

/*
- Crée un nouvel arc du sommet de label "a" vers le sommet de label "b" avec le poids "poids". 
- Les arcs sont rangés dans la liste d’adjacence par indice croissant. 
- Si un arc entre "a" et "b" existe déjà, le poids est mis à jour sans créer de nouvel arc dans la liste d’adjacence.
- Retourne le code d'erreur -1 si "a" n’existe pas, -2 si "b" n’existe pas, -3 si l’allocation de mémoire n’est pas possible.
*/
int ajouterArc(graphe *, int a, int b, int poids);

/*
- Supprime un sommet et les arcs correspondants (i.e., les arcs ayant leur origine ou leur destination ́correspondant à "label")
et libère la mémoire correspondante.
- Retourne le code d'erreur -1 si le sommet n’existe pas.
*/
int supprimerSommet(graphe *, int label);

/*
- Supprime l’arc joignant "a" à "b". Cette fonction libère la mémoire correspondante.
- Retourne le code d'erreur -1 si l’arc n’existe pas
*/
int supprimerArc(graphe *, int a, int b);

/*
- Supprime entièrement un graphe en libérant la mémoire correspondante. 
- Réinitialise le graphe au moyen de la fonction "initialiserGraphe".
*/
void supprimerGraphe(graphe *);

/*
- Affiche à l'écran la valeur des champs nbS, nbA et maxS.
- Affiche également l’ensemble des sommets et pour chacun, la liste des arcs vers les sommets adjacents avec leurs poids respectifs.
*/
void afficherGraphe(graphe *);

#endif
